#
# spec file for package transactional-update
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           transactional-update
Version:        2.20.3
Release:        lp152.2.3.1
Summary:        Transactional Updates with btrfs and snapshots
License:        GPL-2.0-or-later
Group:          System/Base
URL:            https://github.com/openSUSE/transactional-update
Source:         https://github.com/openSUSE/transactional-update/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Source1:        transactional-update.check
Patch1:         0001-Fix-EFI-grub2.patch
BuildRequires:  autoconf-archive
BuildRequires:  automake
BuildRequires:  docbook-xsl-stylesheets
BuildRequires:  fdupes
BuildRequires:  libzypp
BuildRequires:  pkgconfig
BuildRequires:  python3-lxml
BuildRequires:  rpm-devel
BuildRequires:  suse-module-tools
Requires:       attr
Requires:       bc
Requires:       logrotate
Requires:       lsof
# psmisc is needed because of fuser
Requires:       psmisc
Requires:       rsync
Recommends:     rebootmgr

%description
transactional-update is a tool to update a system in an atomic
way with zypper, btrfs and snapshots.

%prep
%setup -q
%patch1 -p1

%build
if [ -x autogen.sh ]; then
	./autogen.sh
fi
%configure --with-doc --docdir=%{_docdir}/%{name}
make %{?_smp_mflags}
# Use "up" for non-rolling releases
%if %{defined sle_version} && %{undefined is_susecasp}
sed -i 's/^UPDATE_METHOD=.*/UPDATE_METHOD=up/' etc/transactional-update.conf
%endif

%install
%make_install
mkdir -p %{buildroot}%{_sysconfdir}/zypp/systemCheck.d/
install -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/zypp/systemCheck.d/
%fdupes %{buildroot}%{_mandir}

%pre
%service_add_pre %{name}.service %{name}.timer create-dirs-from-rpmdb.service

%post
%service_add_post %{name}.service %{name}.timer create-dirs-from-rpmdb.service
%regenerate_initrd_post

%posttrans
%regenerate_initrd_posttrans

%preun
%service_del_preun %{name}.service %{name}.timer create-dirs-from-rpmdb.service

%postun
%service_del_postun %{name}.service %{name}.timer create-dirs-from-rpmdb.service
%regenerate_initrd_post

%files
%license COPYING
%doc NEWS
%doc %{_docdir}/%{name}/transactional-update.txt
%config(noreplace) %{_sysconfdir}/logrotate.d/transactional-update
%{_unitdir}/transactional-update.service
%{_unitdir}/transactional-update.timer
%{_unitdir}/create-dirs-from-rpmdb.service
%{_sbindir}/create_dirs_from_rpmdb
%{_sbindir}/transactional-update
%{_sbindir}/tu-rebuild-kdump-initrd
%dir %{_prefix}%{_sysconfdir}
%{_prefix}%{_sysconfdir}/transactional-update.conf
%dir %{_prefix}/lib/dracut
%dir %{_prefix}/lib/dracut/modules.d
%{_prefix}/lib/dracut/modules.d/50transactional-update/
%{_mandir}/man5/transactional-update.conf.5%{?ext_man}
%{_mandir}/man8/transactional-update.8%{?ext_man}
%{_mandir}/man8/transactional-update.timer.8%{?ext_man}
%{_mandir}/man8/transactional-update.service.8%{?ext_man}
%exclude %{_docdir}/%{name}/*.html

%package zypp-config
Summary:        Zypper rule to prevent uninstallation of transactional-update
Group:          System/Base
BuildArch:      noarch
Requires:       transactional-update

%description zypp-config
Adds a zypper rule to prevent accidental uninstallation of
transactional-update.

%files zypp-config
%config(noreplace) %{_sysconfdir}/zypp/systemCheck.d/transactional-update.check

%changelog
* Tue Aug 18 2020 Ignaz Forster <iforster@suse.com>
- Added 0001-Fix-EFI-grub2.patch:
  Backport of upstream fix for [boo#1162320] (grub2-arm64-efi
  upgrade 2.02-lp151.21.6.1 -> 2.02-lp151.21.9.1 causes system to
  fail reboot with "error: symbol `grub_efi_allocate_any_pages' not
  found.")
* Tue Mar  3 2020 Ignaz Forster <iforster@suse.com>
- Update to version 2.20.3
  - Reintroduce compatibility with RPM < 4.15 for inclusion in
    SLE / Leap 15.2
  - Prepare for removal of /var/lib/alternatives [jsc#SLE-9101]
* Wed Feb 26 2020 Ignaz Forster <iforster@suse.com>
- Update to version 2.20.2
  - Use full names for zypper options [bsc#1164543]
  - Ignore /var/lib/rpm in shadowed file output
  - Optimize create-dirs-from-rpmdb
* Tue Jan  7 2020 Ignaz Forster <iforster@suse.com>
- Update to version 2.20.1
  - Add missing documentation about --continue option
  - Avoid error message about missing fstab file on first snapshot creation.
    [boo#1160213]
* Fri Nov  8 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.20
  - Add the option `--continue` to extend an existing snapshot. This can be
    used to perform multiple operations before rebooting into the new state.
    [gh#openSUSE/transactional-update#16]
  - Make sure the dracut service to print warnings on /etc overlay conflicts
    also runs in the pre-made images.
  - Add "none" reboot method
  - Remove conflicting overlay artifacts in case an existing overlay directory
    will be reused
* Wed Sep 25 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.17
  - Provide network in shell on systems where resolv.conf is pointing to /run
    directly [boo#1151081]
  - Print new snapshot number at the end
  - Always mount /root subvolume when using shell command to preserve history
* Mon Sep  9 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.16
  - Use default command if options, but no command was given [boo#1146116]
  - Make sure only one process appears in `ps` output [boo#1111897]
  - Move update check: If a new repository is added (and
    ZYPPER_AUTO_IMPORT_KEYS is set) adding the new repository key won't fail
    any more
  - Avoid unnecessary snapshots / reboots by detecting zypper operations
    without changes more reliably (e.g. when installing an already installed
    package)
  - Update the manpage accordingly
  - Bugfixes:
  - Ignore commented lines in fstab when checking for shadowed files
  - Avoid warning when copying network config
- Remove Perl dependency
- Building documentation requires Python 3 now
* Wed Jun  5 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.15
  - Move description of interactiveness [boo#1132438]
  - Enable network during update [boo#1137890]
  - Add compatibilty with new overlayfs versions [boo#1140057]
  - Allow updates of the bootloader on EFI systems [boo#1139015]
  - Add support for /boot/writable [boo#1138725]
* Mon Apr 29 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.14.2
  - Prevent unnecessary error message on systems not installed with
    YaST (e.g. KIWI)
* Wed Apr 17 2019 kukuk@suse.de
- Add requires for bc, needed for some calculations
* Mon Mar 25 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.14.1
  - Improve non-root fs changes checker based on feedback
  - Disable snapper's zypper plugin during transactional-update run
- Allow parallel installation with snapper's zypper plugin (useful on
  read-write systems).
* Fri Mar 15 2019 Ignaz Forster <iforster@suse.com>
- Update to version 2.14
  - Warn user if contents of /var have been changed during update
  - Noteworthy: swapped position of upperdir and lowerdir in fstab for better
    readability
  - Major update to the transactional-update guide
* Mon Mar  4 2019 kukuk@suse.de
- Update to version 2.13.2
  - add hooks for telemetrics
* Thu Feb 14 2019 iforster@suse.com
- Update to version 2.13.1
  - Bugfix: Clean up empty /etc snapshot directories again
- Use official release files from GitHub for building
* Thu Feb 14 2019 iforster@suse.com
- Update to version 2.13
  - Fix a serious data loss problem (deleting all files in /etc
    overlays) in case /var/lib/overlay/etc is not part of any
    snapshot any more.
  - Avoid error message in case a system has no /etc/fstab.sys
* Mon Jan 28 2019 iforster@suse.com
- Update to version 2.12
  - Implements support for needs-restart (Fate#326451)
  - Adds easy rollback to last known working snapshot by using "last"
    as the snapshot number
  - Migrate /etc/fstab.sys entires to /etc/fstab for systemd being
    able to create mount units; this will prevent errors on unmount
  - Remove implicit --no-allow-vendor-change - this option can be
    configured in zypper's configuration file now
  - Add support for mounting the /etc overlay file system as a systemd
    unit in /etc/fstab on a read-only system (replacing /etc/fstab.sys
    in the long run)
  - Various bugfixes:
  - Only remove /etc overlays on read-only root file systems
    [boo#1122455]
  - Disable "Persistent" flag in systemd timer unit to prevent
    updates during or immediately after the boot phase
  - Don't set global zypper options during migration - this is not
    supported by the migration plugin
  - Fix error handling when interrupting while rsync is running
  - Filter out security.selinux attributes during rsync
* Sat Nov 17 2018 Jan Engelhardt <jengelh@inai.de>
- Use noun phrase in summary.
* Mon Nov 12 2018 iforster@suse.com
- Update to version 2.11
  - Update man page to explain new /etc overlay handling
  - Add special handling for migrating away from old overlay directory
  - Read default update method for systemd service from config file
- Add package "transactional-update-zypp-config" to prevent accidental
  transactional-update uninstallation on transactional systems
  (boo#1111319)
- Use "up" for systemd service on non-rolling release distributions
  (boo#1111321)
* Thu Oct  4 2018 iforster@suse.com
- Update to version 2.10
  - Introduce /etc overlays
  - Each root file system snapshot now has a dedicated /etc overlay in
    /var/lib/snapshots/<snapshot number>/etc
  - Changes in older snapshots will be visible in newer snapshots (if
    the file hasn't been changed in both snapshots), but not the other
    way around; this makes it possible to roll back to a previous
    snapshot even if the configuration is broken
  - The /etc state of the oldest active snapshot will be synchronized
    into the new snapshot to prevent ever growing overlays
  - Removed creation of /etc backup snapshots
  - Removed complex overlayfs cleanup handling during dracut early boot;
    only thing left is to output a warning if files have been changed
    in old and new overlay after snapshot creation
  - Unused overlays will be removed during "cleanup" operation
  - Add compatibility with snapper >= 0.7.0
  - Correctly detect BTRFS ID when using multiple snapper configurations
  - Fix disappearing prompt in "shell" command when application was updated
  - Prevent accidental rollback on fast reboots (boo#1111786)
  - Use zypper attribute packages-to-change if available (boo#1105992 &
    boo#1097619)
  - Add kexec as reboot alternative
  - Added explicit --with-doc / --without-doc build configure opttions for
    enforcing or excluding documentation build
  - Removed precompiled documentation (boo#1088676)
  - Added more dependency checks to configure script
  - Small improvements to documentation
  - Restored compatibility with older rsync versions not supporting the -x
    filter
  - Small bugfixes for cleanup in error situations
  - Fix possible incorrect detection of snapshots numbers during cleanup
  - Mention parent in snapshot description
- Add dependencies for documentation build (boo#1088676)
* Mon Aug 20 2018 iforster@suse.com
- Update to version 2.6
  - Add --interactive and --non-interactive options to change the default
    interactivity for zypper calls. For previously non-interactive calls (e.g.
    dup) these options makes it possible to pick the correct solution in case
    of conflicts or make sure that a script will never be stuck at an input
    prompt.
  - Add kured reboot support
  - Tries to clean up temporary files and mount points if transactional-update
    quits unexpectedly
  - Remove trancactional-update-helper (replaced by zypper options)
  - Create /var/lib/zypp if not available (boo#1104452)
  - Robustify parsing zypper output (boo#1105992)
  - Various small fixes
- Changed permissions of systemd script again
* Fri Jun 15 2018 iforster@suse.com
- Update to version 2.5
  - Support for overlayfs attributes like trusted.overlay.opaque - this will
    avoid disappearing configuration files on system updates
  - Documentation update
  - Only sync /etc if it's on an overlayfs mount
  - Use correct locale for zypper - thus updating locale specific packages and
    avoiding "empty" updates where transactional-update didn't see the change
- Add dependency to attr
- Fix permissions of systemd script
* Tue Jun  5 2018 kukuk@suse.de
- Update to version 2.4
  - Bug fix release
* Mon May 14 2018 iforster@suse.com
- Update to version 2.3
  - Reformat help text / synopsis
  - More detailled output (use --quiet for less output)
  - Log all error messages into log file
  - Avoid potential lock file race condition on self update
  - Accept package names with whitespace
  - Don't wait for user input during self-update
  - Fix broken error message output when creating new snapshot without reboot
  - Fix erroneous mounting of stale grub mounts
* Wed May  2 2018 iforster@suse.com
- Update to version 2.2
  - Fix security issue in self update [boo#1091316]
  - Create dirs in /var more reliably by sorting the list
- Rebuild initrd (needed since 2.0)
* Thu Apr 26 2018 kukuk@suse.de
- Update to version 2.1
  - Cleanup the code cleaning up /etc
* Tue Apr 24 2018 kukuk@suse.de
- Update to version 2.0
  - Create missing directories from rpm database during boot
  - Merge /etc overlay with root subvolume during update
* Fri Apr 20 2018 kukuk@suse.de
- Update to version 1.29
  - Implement self-update
  - Disable optical media on dup
  - Ignore certain zypper return codes
* Tue Apr  3 2018 rbrown@suse.com
- Add conflicts with snapper-zypp-plugin
* Sat Mar 17 2018 rbrown@suse.com
- Require perl-HTML-Parser (needed since 1.26) [boo#1085764]
* Mon Mar 12 2018 kukuk@suse.de
- Update to version 1.28
  - Adjust for latest grub2
* Tue Mar  6 2018 kukuk@suse.de
- Update to version 1.27
  - Fallback to snapper without dbus support if dbus fails
  - Add workaround for update-alternatives
  - Add support for /usr/etc/transactional-update.conf
* Fri Feb 23 2018 kukuk@suse.com
- Use %%license instead of %%doc [bsc#1082318]
* Wed Jan 31 2018 kukuk@suse.de
- Update to version 1.26
  - Auto accept repo keys config option
  - Detect broken snapshots after abort
  - Ignore user deleted snapshots
  - Include all error messages in main log file
  - Add configuration file
  - Correctly implement migration of major OS versions
  - Default reboot strategy is configuration option
  - Salt option is deprecated
* Tue Jan  9 2018 kukuk@suse.de
- Update to version 1.25
  - preliminary SELinux support
  - support for seperate /var subvolume
* Tue Dec  5 2017 kukuk@suse.de
- Update to version 1.24
  - Don't save unused snapshots if we may delete them again.
    Partly fixes [bsc#1071038]
  - Copy passwd, group and shadow to /usr/etc if modified
* Fri Nov 17 2017 kukuk@suse.de
- Update to version 1.23
  - Fix re-registration in rollback case
  - Fix problems with updating the grub2 bootloader
  - Add support for migration of products with SCC/SMT
  - Re-add support to re-write bootloader
  [bsc#1068933]
* Tue Nov  7 2017 kukuk@suse.de
- Update to version 1.22
  - Rewrite cleanup code to be more aggressive
* Fri Oct 27 2017 kukuk@suse.de
- Update to version 1.21
  - Reset salt grains even if no update was found
* Wed Sep 13 2017 kukuk@suse.de
- Update to version 1.20
  - Improve documentation
  - Add pkg update option
* Fri Jun 30 2017 kukuk@suse.de
- Update to version 1.19
  - Add 'shell' option for debugging and testing
* Wed Jun 28 2017 kukuk@suse.de
- Update to version 1.18
  - Auto accept product license in non-interactive mode
* Tue Jun 27 2017 kukuk@suse.de
- Update to version 1.17
  - copy modified bootloader config into snapshot
  - unit file cleanup, no change of behavior
* Mon Jun 26 2017 kukuk@suse.de
- Update to version 1.16
  - internal changes [bsc#1045942]
* Thu Jun  1 2017 kukuk@suse.de
- Update to version 1.15
  - Rename option ptf to pkg
  - Add option to set salt grains
* Fri Apr 21 2017 kukuk@suse.de
- Update to version 1.14
  - Fix check of return values of commands in pipes [bsc#1034900]
* Tue Mar 21 2017 kukuk@suse.de
- Update to version 1.12
  - minor bug fixes
* Thu Mar  2 2017 kukuk@suse.de
- Update to version 1.11
  - use --no-allow-vendor-change with "zypper dup"
  - update manual page
* Mon Feb 13 2017 kukuk@suse.de
- Update to version 1.10
  - implement and document rollback option
* Tue Feb  7 2017 kukuk@suse.de
- Update to version 1.9
  - fix umounting of special filesystems and subvolumes
  - fix interactive installation of PTFs [bsc#1023026]
  - fix dependencies of systemd service
* Wed Feb  1 2017 kukuk@suse.de
- Add Requires for lsof and fuser
* Wed Feb  1 2017 kukuk@suse.de
- Update to version 1.8
  - fix references in html docu
  - build kdump initrd if we build standard initrd, too
* Mon Jan 30 2017 kukuk@suse.de
- Update to version 1.7
  - build html docu, too
* Sun Jan 29 2017 kukuk@suse.de
- Update to version 1.6
  - Add support for kdump [bsc#1018098]
* Thu Jan 19 2017 kukuk@suse.de
- Update to version 1.5
  - Install PTFs in interactive mode
  - Sync state file between current and new snapshot
* Tue Jan 17 2017 kukuk@suse.de
- Update to version 1.4
  - Fix wrong error messages
  - More debug output if umount fails
  - Try a second time if umount fails
* Mon Dec 19 2016 kukuk@suse.de
- Update to version 1.3
  - Add initrd option
* Thu Dec 15 2016 kukuk@suse.de
- Do some spec file cleanup
* Mon Dec 12 2016 kukuk@suse.de
- Update to version 1.2
  - suppport ro-subvolume with overlayfs
* Tue Dec  6 2016 kukuk@suse.de
- Update to version 1.1
  - bug fix release
  - implement lock
* Wed Nov 23 2016 kukuk@suse.de
- Update to version 1.0
  - implement and document PTF options
* Mon Nov 21 2016 kukuk@suse.de
- Update to version 0.99
  - Support for rebootmgr
  - Write real manual page
* Mon Nov 21 2016 kukuk@suse.de
- Add empty directory
  /etc/systemd/system/transactional-update.timer.d/
  so to make it easier to override transactional-update.timer
  entries.
* Thu Nov 17 2016 kukuk@suse.de
- Update to version 0.90
  - Finish normal options
  - Implement support for PTFs
* Sun Nov  6 2016 kukuk@suse.com
- Disable restart on update for post install scripts
* Sun Nov  6 2016 kukuk@suse.de
- Implement activating the new snapshot as new root filesystem
* Fri Nov  4 2016 kukuk@suse.de
- Initial version
